@@ -17,18 +17,22 @@ import argparse
 import configparser
 from datetime import datetime
 import io
-import json
-import os
 from pathlib import Path
 import re
 import shutil
 import subprocess
 import tempfile
-from typing import Iterable, List
+from typing import Iterable
 
 import setuptools  # type: ignore
 
-from pw_build.python_package import PythonPackage
+try:
+    from pw_build.python_package import (PythonPackage, load_packages,
+                                         change_working_dir)
+except ImportError:
+    # Load from python_package from this directory if pw_build is not available.
+    from python_package import (  # type: ignore
+        PythonPackage, load_packages, change_working_dir)
 
 
 def _parse_args():
@@ -88,7 +92,7 @@ def get_current_git_sha() -> str:
 
 
 def get_current_date() -> str:
-    return datetime.now().strftime('%Y%m%d%H%M')
+    return datetime.now().strftime('%Y%m%d%H%M%S')
 
 
 class UnexpectedConfigSection(Exception):
@@ -149,16 +153,16 @@ def update_config_with_packages(
     included_packages = [pkg.package_name for pkg in python_packages]
 
     for pkg in python_packages:
-        assert pkg.config
+        # Skip this package if no setup.cfg is defined.
+        if not pkg.config:
+            continue
 
         # Collect install_requires
         if pkg.config.has_option('options', 'install_requires'):
             existing_requires = config['options'].get('install_requires', '\n')
-            # Requires are delimited by newlines or semicolons.
-            # Split existing list on either one.
-            this_requires = re.split(r' *[\n;] *',
-                                     pkg.config['options']['install_requires'])
-            new_requires = existing_requires.splitlines() + this_requires
+
+            new_requires = existing_requires.splitlines()
+            new_requires += pkg.install_requires_entries()
             # Remove requires already included in this merged config.
             new_requires = [
                 line for line in new_requires
@@ -203,22 +207,43 @@ def write_config(
     setup_cfg_file.write_text(comment_block_text + setup_cfg_text.getvalue())
 
 
-def load_packages(input_list_files: Iterable[Path]) -> List[PythonPackage]:
-    """Load Python package metadata and configs."""
+def setuptools_build_with_base(pkg: PythonPackage,
+                               build_base: Path,
+                               include_tests: bool = False) -> Path:
+    """Run setuptools build for this package."""
 
-    packages = []
-    for input_path in input_list_files:
+    # If there is no setup_dir or setup_sources, just copy this packages
+    # source files.
+    if not pkg.setup_dir:
+        pkg.copy_sources_to(build_base)
+        return build_base
+    # Create the lib install dir in case it doesn't exist.
+    lib_dir_path = build_base / 'lib'
+    lib_dir_path.mkdir(parents=True, exist_ok=True)
 
-        with input_path.open() as input_file:
-            # Each line contains the path to a json file.
-            for json_file in input_file.readlines():
-                # Load the json as a dict.
-                json_file_path = Path(json_file.strip()).resolve()
-                with json_file_path.open() as json_fp:
-                    json_dict = json.load(json_fp)
+    starting_directory = Path.cwd()
+    # cd to the location of setup.py
+    with change_working_dir(pkg.setup_dir):
+        # Run build with temp build-base location
+        # Note: New files will be placed inside lib_dir_path
+        setuptools.setup(script_args=[
+            'build',
+            '--force',
+            '--build-base',
+            str(build_base),
+        ])
 
-                packages.append(PythonPackage.from_dict(**json_dict))
-    return packages
+        new_pkg_dir = lib_dir_path / pkg.package_name
+        # If tests should be included, copy them to the tests dir
+        if include_tests and pkg.tests:
+            test_dir_path = new_pkg_dir / 'tests'
+            test_dir_path.mkdir(parents=True, exist_ok=True)
+
+            for test_source_path in pkg.tests:
+                shutil.copy(starting_directory / test_source_path,
+                            test_dir_path)
+
+    return lib_dir_path
 
 
 def build_python_tree(python_packages: Iterable[PythonPackage],
@@ -226,9 +251,6 @@ def build_python_tree(python_packages: Iterable[PythonPackage],
                       include_tests: bool = False) -> None:
     """Install PythonPackages to a destination directory."""
 
-    # Save the current out directory
-    out_dir = Path.cwd()
-
     # Create the root destination directory.
     destination_path = tree_destination_dir.resolve()
     # Delete any existing files
@@ -238,33 +260,10 @@ def build_python_tree(python_packages: Iterable[PythonPackage],
     # Define a temporary location to run setup.py build in.
     with tempfile.TemporaryDirectory() as build_base_name:
         build_base = Path(build_base_name)
-        lib_dir_path = build_base / 'lib'
 
         for pkg in python_packages:
-            # Create the temp install dir
-            lib_dir_path.mkdir(parents=True, exist_ok=True)
-
-            # cd to the location of setup.py
-            setup_dir_path = out_dir / pkg.setup_dir
-            os.chdir(setup_dir_path)
-            # Run build with temp build-base location
-            # Note: New files will be placed inside lib_dir_path
-            setuptools.setup(script_args=[
-                'build',
-                '--force',
-                '--build-base',
-                str(build_base),
-            ])
-
-            new_pkg_dir = lib_dir_path / pkg.package_name
-
-            # If tests should be included, copy them to the tests dir
-            if include_tests and pkg.tests:
-                test_dir_path = new_pkg_dir / 'tests'
-                test_dir_path.mkdir(parents=True, exist_ok=True)
-
-                for test_source_path in pkg.tests:
-                    shutil.copy(out_dir / test_source_path, test_dir_path)
+            lib_dir_path = setuptools_build_with_base(
+                pkg, build_base, include_tests=include_tests)
 
             # Move installed files from the temp build-base into
             # destination_path.
@@ -277,9 +276,6 @@ def build_python_tree(python_packages: Iterable[PythonPackage],
             # Clean build base lib folder for next install
             shutil.rmtree(lib_dir_path, ignore_errors=True)
 
-    # cd back to out directory
-    os.chdir(out_dir)
-
 
 def copy_extra_files(extra_file_strings: Iterable[str]) -> None:
     """Copy extra files to their destinations."""
