@@ -21,45 +21,15 @@ The stdout of this script is meant to be executed by the invoking shell.
 
 from __future__ import print_function
 
-import argparse
+import hashlib
 import json
 import os
-import platform
+import platform as platform_module
 import re
 import subprocess
 import sys
 
 
-def parse(argv=None):
-    """Parse arguments."""
-
-    script_root = os.path.join(os.environ['PW_ROOT'], 'pw_env_setup', 'py',
-                               'pw_env_setup', 'cipd_setup')
-    git_root = subprocess.check_output(
-        ('git', 'rev-parse', '--show-toplevel'),
-        cwd=script_root,
-    ).decode('utf-8').strip()
-
-    parser = argparse.ArgumentParser(description=__doc__.splitlines()[0])
-    parser.add_argument(
-        '--install-dir',
-        dest='root_install_dir',
-        default=os.path.join(git_root, '.cipd'),
-    )
-    parser.add_argument('--package-file',
-                        dest='package_files',
-                        metavar='PACKAGE_FILE',
-                        action='append')
-    parser.add_argument('--cipd',
-                        default=os.path.join(script_root, 'wrapper.py'))
-    parser.add_argument('--cache-dir',
-                        default=os.environ.get(
-                            'CIPD_CACHE_DIR',
-                            os.path.expanduser('~/.cipd-cache-dir')))
-
-    return parser.parse_args(argv)
-
-
 def check_auth(cipd, package_files, spin):
     """Check have access to CIPD pigweed directory."""
 
@@ -148,23 +118,30 @@ def check_auth(cipd, package_files, spin):
     return True
 
 
-def _platform():
+def platform():
     osname = {
         'darwin': 'mac',
         'linux': 'linux',
         'windows': 'windows',
-    }[platform.system().lower()]
+    }[platform_module.system().lower()]
 
-    if platform.machine().startswith(('aarch64', 'armv8')):
+    if platform_module.machine().startswith(('aarch64', 'armv8')):
         arch = 'arm64'
-    elif platform.machine() == 'x86_64':
+    elif platform_module.machine() == 'x86_64':
         arch = 'amd64'
-    elif platform.machine() == 'i686':
+    elif platform_module.machine() == 'i686':
         arch = 'i386'
     else:
-        arch = platform.machine()
+        arch = platform_module.machine()
 
-    return '{}-{}'.format(osname, arch).lower()
+    platform_arch = '{}-{}'.format(osname, arch).lower()
+
+    # Support `mac-arm64` through Rosetta until `mac-arm64` binaries are ready
+    if platform_arch == 'mac-arm64' and os.getenv(
+            'PW_BOOTSTRAP_USE_ROSETTA', 'false').lower() in ('true', 't', '1'):
+        return 'mac-amd64'
+
+    return platform_arch
 
 
 def all_package_files(env_vars, package_files):
@@ -202,9 +179,19 @@ def all_package_files(env_vars, package_files):
     return result
 
 
-def write_ensure_file(package_file, ensure_file):
-    with open(package_file, 'r') as ins:
-        packages = json.load(ins).get('packages', ())
+def write_ensure_file(package_files, ensure_file):
+    packages = []
+
+    for package_file in package_files:
+        name = package_file_name(package_file)
+        with open(package_file, 'r') as ins:
+            file_packages = json.load(ins).get('packages', ())
+            for package in file_packages:
+                if 'subdir' in package:
+                    package['subdir'] = os.path.join(name, package['subdir'])
+                else:
+                    package['subdir'] = name
+            packages.extend(file_packages)
 
     with open(ensure_file, 'w') as outs:
         outs.write('$VerifiedPlatform linux-amd64\n'
@@ -214,13 +201,17 @@ def write_ensure_file(package_file, ensure_file):
         for pkg in packages:
             # If this is a new-style package manifest platform handling must
             # be done here instead of by the cipd executable.
-            if 'platforms' in pkg and _platform() not in pkg['platforms']:
+            if 'platforms' in pkg and platform() not in pkg['platforms']:
                 continue
 
             outs.write('@Subdir {}\n'.format(pkg.get('subdir', '')))
             outs.write('{} {}\n'.format(pkg['path'], ' '.join(pkg['tags'])))
 
 
+def package_file_name(package_file):
+    return os.path.basename(os.path.splitext(package_file)[0])
+
+
 def package_installation_path(root_install_dir, package_file):
     """Returns the package installation path.
 
@@ -228,8 +219,8 @@ def package_installation_path(root_install_dir, package_file):
       root_install_dir: The CIPD installation directory.
       package_file: The path to the .json package definition file.
     """
-    return os.path.join(root_install_dir,
-                        os.path.basename(os.path.splitext(package_file)[0]))
+    return os.path.join(root_install_dir, 'packages',
+                        package_file_name(package_file))
 
 
 def update(
@@ -239,87 +230,114 @@ def update(
     cache_dir,
     env_vars=None,
     spin=None,
+    trust_hash=False,
 ):
     """Grab the tools listed in ensure_files."""
 
     package_files = all_package_files(env_vars, package_files)
 
-    if not check_auth(cipd, package_files, spin):
-        return False
-
     # TODO(mohrr) use os.makedirs(..., exist_ok=True).
     if not os.path.isdir(root_install_dir):
         os.makedirs(root_install_dir)
 
+    # This file is read by 'pw doctor' which needs to know which package files
+    # were used in the environment.
+    package_files_file = os.path.join(root_install_dir,
+                                      '_all_package_files.json')
+    with open(package_files_file, 'w') as outs:
+        json.dump(package_files, outs, indent=2)
+
     if env_vars:
         env_vars.prepend('PATH', root_install_dir)
         env_vars.set('PW_CIPD_INSTALL_DIR', root_install_dir)
         env_vars.set('CIPD_CACHE_DIR', cache_dir)
 
     pw_root = None
+
     if env_vars:
         pw_root = env_vars.get('PW_ROOT', None)
     if not pw_root:
         pw_root = os.environ['PW_ROOT']
 
-    # Run cipd for each json file.
-    for package_file in package_files:
-        if os.path.splitext(package_file)[1] == '.ensure':
-            ensure_file = package_file
-        else:
-            ensure_file = os.path.join(
-                root_install_dir,
-                os.path.basename(
-                    os.path.splitext(package_file)[0] + '.ensure'))
-            write_ensure_file(package_file, ensure_file)
+    ensure_file = os.path.join(root_install_dir, 'packages.ensure')
+    write_ensure_file(package_files, ensure_file)
 
-        install_dir = package_installation_path(root_install_dir, package_file)
+    install_dir = os.path.join(root_install_dir, 'packages')
 
-        name = os.path.basename(install_dir)
+    cmd = [
+        cipd,
+        'ensure',
+        '-ensure-file', ensure_file,
+        '-root', install_dir,
+        '-log-level', 'debug',
+        '-json-output', os.path.join(root_install_dir, 'packages.json'),
+        '-cache-dir', cache_dir,
+        '-max-threads', '0',  # 0 means use CPU count.
+    ]  # yapf: disable
 
-        cmd = [
-            cipd,
-            'ensure',
-            '-ensure-file', ensure_file,
-            '-root', install_dir,
-            '-log-level', 'debug',
-            '-json-output',
-            os.path.join(root_install_dir, '{}-output.json'.format(name)),
-            '-cache-dir', cache_dir,
-            '-max-threads', '0',  # 0 means use CPU count.
-        ]  # yapf: disable
+    hasher = hashlib.sha256()
+    encoded = '\0'.join(cmd)
+    if hasattr(encoded, 'encode'):
+        encoded = encoded.encode()
+    hasher.update(encoded)
+    with open(ensure_file, 'rb') as ins:
+        hasher.update(ins.read())
+    digest = hasher.hexdigest()
 
-        # TODO(pwbug/135) Use function from common utility module.
-        log = os.path.join(root_install_dir, '{}.log'.format(name))
-        try:
-            with open(log, 'w') as outs:
-                print(*cmd, file=outs)
-                subprocess.check_call(cmd,
-                                      stdout=outs,
-                                      stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError:
-            with open(log, 'r') as ins:
-                sys.stderr.write(ins.read())
-                raise
+    with open(os.path.join(root_install_dir, 'hash.log'), 'w') as hashlog:
+        print('calculated digest:', digest, file=hashlog)
 
-        # Set environment variables so tools can later find things under, for
-        # example, 'share'.
-        if env_vars:
+        hash_file = os.path.join(root_install_dir, 'packages.sha256')
+        print('hash file path:', hash_file, file=hashlog)
+        print('exists:', os.path.isfile(hash_file), file=hashlog)
+        print('trust_hash:', trust_hash, file=hashlog)
+        if trust_hash and os.path.isfile(hash_file):
+            with open(hash_file, 'r') as ins:
+                digest_file = ins.read().strip()
+                print('contents:', digest_file, file=hashlog)
+                print('equal:', digest == digest_file, file=hashlog)
+                if digest == digest_file:
+                    return True
+
+    if not check_auth(cipd, package_files, spin):
+        return False
+
+    # TODO(pwbug/135) Use function from common utility module.
+    log = os.path.join(root_install_dir, 'packages.log')
+    try:
+        with open(log, 'w') as outs:
+            print(*cmd, file=outs)
+            subprocess.check_call(cmd, stdout=outs, stderr=subprocess.STDOUT)
+    except subprocess.CalledProcessError:
+        with open(log, 'r') as ins:
+            sys.stderr.write(ins.read())
+            raise
+
+    with open(hash_file, 'w') as outs:
+        print(digest, file=outs)
+
+    # Set environment variables so tools can later find things under, for
+    # example, 'share'.
+    if env_vars:
+        for package_file in package_files:
+            name = package_file_name(package_file)
+            file_install_dir = os.path.join(install_dir, name)
             # Some executables get installed at top-level and some get
-            # installed under 'bin'.
-            env_vars.prepend('PATH', install_dir)
-            env_vars.prepend('PATH', os.path.join(install_dir, 'bin'))
+            # installed under 'bin'. A small number of old packages prefix the
+            # entire tree with the platform (e.g., chromium/third_party/tcl).
+            for bin_dir in (
+                    file_install_dir,
+                    os.path.join(file_install_dir, 'bin'),
+                    os.path.join(file_install_dir, platform(), 'bin'),
+            ):
+                if os.path.isdir(bin_dir):
+                    env_vars.prepend('PATH', bin_dir)
             env_vars.set('PW_{}_CIPD_INSTALL_DIR'.format(name.upper()),
-                         install_dir)
+                         file_install_dir)
 
             # Windows has its own special toolchain.
             if os.name == 'nt':
-                env_vars.prepend('PATH',
-                                 os.path.join(install_dir, 'mingw64', 'bin'))
+                env_vars.prepend(
+                    'PATH', os.path.join(file_install_dir, 'mingw64', 'bin'))
 
     return True
-
-
-if __name__ == '__main__':
-    update(**vars(parse()))
-    sys.exit(0)
