@@ -172,9 +172,9 @@ class MissingSubmodulesError(Exception):
 class EnvSetup(object):
     """Run environment setup for Pigweed."""
     def __init__(self, pw_root, cipd_cache_dir, shell_file, quiet, install_dir,
-                 virtualenv_root, strict, virtualenv_gn_out_dir, json_file,
-                 project_root, config_file, use_existing_cipd,
-                 use_pinned_pip_packages):
+                 strict, virtualenv_gn_out_dir, json_file, project_root,
+                 config_file, use_existing_cipd, check_submodules,
+                 use_pinned_pip_packages, cipd_only, trust_cipd_hash):
         self._env = environment.Environment()
         self._project_root = project_root
         self._pw_root = pw_root
@@ -185,9 +185,10 @@ class EnvSetup(object):
         self._is_windows = os.name == 'nt'
         self._quiet = quiet
         self._install_dir = install_dir
-        self._virtualenv_root = (virtualenv_root
-                                 or os.path.join(install_dir, 'pigweed-venv'))
+        self._virtualenv_root = os.path.join(self._install_dir, 'pigweed-venv')
         self._strict = strict
+        self._cipd_only = cipd_only
+        self._trust_cipd_hash = trust_cipd_hash
 
         if os.path.isfile(shell_file):
             os.unlink(shell_file)
@@ -206,15 +207,17 @@ class EnvSetup(object):
         self._pw_packages = []
         self._root_variable = None
 
+        self._check_submodules = check_submodules
+
+        self._json_file = json_file
+        self._gni_file = None
+
         self._config_file_name = getattr(config_file, 'name', 'config file')
+        self._env.set('_PW_ENVIRONMENT_CONFIG_FILE', self._config_file_name)
         if config_file:
             self._parse_config_file(config_file)
 
-        self._check_submodules()
-
-        self._json_file = json_file
-        if not self._json_file:
-            self._json_file = os.path.join(self._install_dir, 'actions.json')
+        self._check_submodule_presence()
 
         self._use_existing_cipd = use_existing_cipd
         self._virtualenv_gn_out_dir = virtualenv_gn_out_dir
@@ -258,6 +261,11 @@ class EnvSetup(object):
 
         self._root_variable = config.pop('root_variable', None)
 
+        if 'json_file' in config:
+            self._json_file = config.pop('json_file')
+
+        self._gni_file = config.pop('gni_file', None)
+
         self._optional_submodules.extend(config.pop('optional_submodules', ()))
         self._required_submodules.extend(config.pop('required_submodules', ()))
 
@@ -299,13 +307,16 @@ class EnvSetup(object):
             raise ConfigFileError('unrecognized option in {}: "{}"'.format(
                 self._config_file_name, next(iter(config))))
 
-    def _check_submodules(self):
+    def _check_submodule_presence(self):
         unitialized = set()
 
         # Don't check submodule presence if using the Android Repo Tool.
         if os.path.isdir(os.path.join(self._project_root, '.repo')):
             return
 
+        if not self._check_submodules:
+            return
+
         cmd = ['git', 'submodule', 'status', '--recursive']
 
         for line in subprocess.check_output(
@@ -351,6 +362,15 @@ class EnvSetup(object):
 
             raise MissingSubmodulesError(', '.join(sorted(missing)))
 
+    def _write_gni_file(self):
+        gni_file = os.path.join(self._project_root, 'build_overrides',
+                                'pigweed_environment.gni')
+        if self._gni_file:
+            gni_file = os.path.join(self._project_root, self._gni_file)
+
+        with open(gni_file, 'w') as outs:
+            self._env.gni(outs, self._project_root)
+
     def _log(self, *args, **kwargs):
         # Not using logging module because it's awkward to flush a log handler.
         if self._quiet:
@@ -378,6 +398,9 @@ class EnvSetup(object):
         if self._is_windows:
             steps.append(("Windows scripts", self.win_scripts))
 
+        if self._cipd_only:
+            steps = [('CIPD package manager', self.cipd)]
+
         self._log(
             Color.bold('Downloading and installing packages into local '
                        'source directory:\n'))
@@ -438,6 +461,12 @@ Then use `set +x` to go back to normal.
             with open(actions_json, 'w') as outs:
                 self._env.json(outs)
 
+            # This file needs to be written after the CIPD step and before the
+            # Python virtualenv step. It also needs to be rewritten after the
+            # Python virtualenv step, so it's easiest to just write it after
+            # every step.
+            self._write_gni_file()
+
         self._log('')
         self._env.echo('')
 
@@ -453,6 +482,10 @@ Then use `set +x` to go back to normal.
             Color.bold('Environment looks good, you are ready to go!'))
         self._env.echo()
 
+        # Don't write new files if all we did was update CIPD packages.
+        if self._cipd_only:
+            return 0
+
         with open(self._shell_file, 'w') as outs:
             self._env.write(outs)
 
@@ -475,9 +508,10 @@ Then use `set +x` to go back to normal.
             outs.write(
                 json.dumps(config, indent=4, separators=(',', ': ')) + '\n')
 
-        if self._json_file is not None:
-            with open(self._json_file, 'w') as outs:
-                self._env.json(outs)
+        json_file = (self._json_file
+                     or os.path.join(self._install_dir, 'actions.json'))
+        with open(json_file, 'w') as outs:
+            self._env.json(outs)
 
         return 0
 
@@ -513,7 +547,8 @@ Then use `set +x` to go back to normal.
                                   package_files=package_files,
                                   cache_dir=self._cipd_cache_dir,
                                   env_vars=self._env,
-                                  spin=spin):
+                                  spin=spin,
+                                  trust_hash=self._trust_cipd_hash):
             return result(_Result.Status.FAILED)
 
         return result(_Result.Status.DONE)
@@ -641,6 +676,14 @@ def parse(argv=None):
                                os.path.expanduser('~/.cipd-cache-dir')),
     )
 
+    parser.add_argument(
+        '--trust-cipd-hash',
+        action='store_true',
+        help='Only run the cipd executable if the ensure file or command-line '
+        'has changed. Defaults to false since files could have been deleted '
+        'from the installation directory and cipd would add them back.',
+    )
+
     parser.add_argument(
         '--shell-file',
         help='Where to write the file for shells to source.',
@@ -673,19 +716,7 @@ def parse(argv=None):
               'packages with GN; defaults to a unique path in the environment '
               'directory.'))
 
-    parser.add_argument(
-        '--virtualenv-root',
-        help=('Root of virtualenv directory. Default: '
-              '<install_dir>/pigweed-venv'),
-        default=None,
-    )
-
-    parser.add_argument(
-        '--json-file',
-        help=('Dump environment variable operations to a JSON file. Default: '
-              '<install_dir>/actions.json'),
-        default=None,
-    )
+    parser.add_argument('--json-file', help=argparse.SUPPRESS, default=None)
 
     parser.add_argument(
         '--use-existing-cipd',
@@ -706,6 +737,19 @@ def parse(argv=None):
         action='store_false',
     )
 
+    parser.add_argument(
+        '--cipd-only',
+        help='Skip non-CIPD steps.',
+        action='store_true',
+    )
+
+    parser.add_argument(
+        '--skip-submodule-check',
+        help='Skip checking for submodule presence.',
+        dest='check_submodules',
+        action='store_false',
+    )
+
     args = parser.parse_args(argv)
 
     return args
