@@ -13,19 +13,17 @@
 # the License.
 """Utilities for using HDLC with pw_rpc."""
 
-import collections
 from concurrent.futures import ThreadPoolExecutor
 import io
 import logging
 from queue import SimpleQueue
-import random
 import sys
 import threading
 import time
 import socket
 import subprocess
-from typing import (Any, BinaryIO, Callable, Deque, Dict, Iterable, List,
-                    NoReturn, Optional, Sequence, Tuple, Union)
+from typing import (Any, BinaryIO, Callable, Dict, Iterable, List, NoReturn,
+                    Optional, Sequence, Union)
 
 from pw_protobuf_compiler import python_protos
 import pw_rpc
@@ -139,7 +137,8 @@ class HdlcRpcClient:
                  output: Callable[[bytes], Any] = write_to_file,
                  client_impl: pw_rpc.client.ClientImpl = None,
                  *,
-                 _incoming_packet_filter_for_testing: '_PacketFilter' = None):
+                 _incoming_packet_filter_for_testing: pw_rpc.
+                 ChannelManipulator = None):
         """Creates an RPC client configured to communicate using HDLC.
 
         Args:
@@ -159,10 +158,13 @@ class HdlcRpcClient:
         self.client = pw_rpc.Client.from_modules(client_impl, channels,
                                                  self.protos.modules())
 
-        self._test_filter = _incoming_packet_filter_for_testing
+        rpc_output: Callable[[bytes], Any] = self._handle_rpc_packet
+        if _incoming_packet_filter_for_testing is not None:
+            _incoming_packet_filter_for_testing.send_packet = rpc_output
+            rpc_output = _incoming_packet_filter_for_testing
 
         frame_handlers: FrameHandlers = {
-            DEFAULT_ADDRESS: self._handle_rpc_packet,
+            DEFAULT_ADDRESS: lambda frame: rpc_output(frame.data),
             STDOUT_ADDRESS: lambda frame: output(frame.data),
         }
 
@@ -184,15 +186,12 @@ class HdlcRpcClient:
 
         return self.client.channel(channel_id).rpcs
 
-    def _handle_rpc_packet(self, frame: Frame) -> None:
-        if self._test_filter and not self._test_filter.keep_packet(frame.data):
-            return
-
-        if not self.client.process_packet(frame.data):
-            _LOG.error('Packet not handled by RPC client: %s', frame.data)
+    def _handle_rpc_packet(self, packet: bytes) -> None:
+        if not self.client.process_packet(packet):
+            _LOG.error('Packet not handled by RPC client: %s', packet)
 
 
-def _try_connect(sock: socket.socket, port: int, attempts: int = 10) -> None:
+def _try_connect(port: int, attempts: int = 10) -> socket.socket:
     """Tries to connect to the specified port up to the given number of times.
 
     This is helpful when connecting to a process that was started by this
@@ -205,9 +204,11 @@ def _try_connect(sock: socket.socket, port: int, attempts: int = 10) -> None:
         time.sleep(0.001)
 
         try:
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             sock.connect(('localhost', port))
-            return
+            return sock
         except ConnectionRefusedError:
+            sock.close()
             if attempts <= 0:
                 raise
 
@@ -218,21 +219,13 @@ class SocketSubprocess:
         self._server_process = subprocess.Popen(command, stdin=subprocess.PIPE)
         self.stdin = self._server_process.stdin
 
-        sock = None
-
         try:
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            _try_connect(sock, port)
+            self.socket: socket.socket = _try_connect(port)  # 🧦
         except:
-            if sock:
-                sock.close()
-
             self._server_process.terminate()
             self._server_process.communicate()
             raise
 
-        self.socket: socket.socket = sock  # 🧦
-
     def close(self) -> None:
         try:
             self.socket.close()
@@ -247,82 +240,21 @@ class SocketSubprocess:
         self.close()
 
 
-class _PacketFilter:
-    """Determines if a packet should be kept or dropped for testing purposes."""
-    _Action = Callable[[int], Tuple[bool, bool]]
-    _KEEP = lambda _: (True, False)
-    _DROP = lambda _: (False, False)
-
-    def __init__(self, name: str) -> None:
-        self.name = name
-        self.packet_count = 0
-        self._actions: Deque[_PacketFilter._Action] = collections.deque()
-
-    def reset(self) -> None:
-        self.packet_count = 0
-        self._actions.clear()
-
-    def keep(self, count: int) -> None:
-        """Keeps the next count packets."""
-        self._actions.extend(_PacketFilter._KEEP for _ in range(count))
-
-    def drop(self, count: int) -> None:
-        """Drops the next count packets."""
-        self._actions.extend(_PacketFilter._DROP for _ in range(count))
-
-    def drop_every(self, every: int) -> None:
-        """Drops every Nth packet forever."""
-        self._actions.append(lambda count: (count % every != 0, True))
-
-    def randomly_drop(self, one_in: int, gen: random.Random) -> None:
-        """Drops packets randomly forever."""
-        self._actions.append(lambda _: (gen.randrange(one_in) != 0, True))
-
-    def keep_packet(self, packet: bytes) -> bool:
-        """Returns whether the provided packet should be kept or dropped."""
-        self.packet_count += 1
-
-        if not self._actions:
-            return True
-
-        keep, repeat = self._actions[0](self.packet_count)
-
-        if not repeat:
-            self._actions.popleft()
-
-        if not keep:
-            _LOG.debug('Dropping %s packet %d for testing: %s', self.name,
-                       self.packet_count, packet)
-        return keep
-
-
-class _TestChannelOutput:
-    def __init__(self, send: Callable[[bytes], Any]) -> None:
-        self._send = send
-        self.packets = _PacketFilter('outgoing RPC')
-
-    def __call__(self, data: bytes) -> None:
-        if self.packets.keep_packet(data):
-            self._send(data)
-
-
 class HdlcRpcLocalServerAndClient:
     """Runs an RPC server in a subprocess and connects to it over a socket.
 
     This can be used to run a local RPC server in an integration test.
     """
-    def __init__(self,
-                 server_command: Sequence,
-                 port: int,
-                 protos: PathsModulesOrProtoLibrary,
-                 *,
-                 for_testing: bool = False) -> None:
-        """Creates a new HdlcRpcLocalServerAndClient.
-
-        If for_testing=True, the HdlcRpcLocalServerAndClient will have
-        outgoing_packets and incoming_packets _PacketFilter members that can be
-        used to program packet loss for testing purposes.
-        """
+    def __init__(
+        self,
+        server_command: Sequence,
+        port: int,
+        protos: PathsModulesOrProtoLibrary,
+        *,
+        incoming_processor: Optional[pw_rpc.ChannelManipulator] = None,
+        outgoing_processor: Optional[pw_rpc.ChannelManipulator] = None
+    ) -> None:
+        """Creates a new HdlcRpcLocalServerAndClient."""
 
         self.server = SocketSubprocess(server_command, port)
 
@@ -333,20 +265,18 @@ class HdlcRpcLocalServerAndClient:
         self.output = io.BytesIO()
 
         self.channel_output: Any = self.server.socket.sendall
-        if for_testing:
-            self.channel_output = _TestChannelOutput(self.channel_output)
-            self.outgoing_packets = self.channel_output.packets
-            self.incoming_packets = _PacketFilter('incoming RPC')
-            incoming_filter: Optional[_PacketFilter] = self.incoming_packets
-        else:
-            incoming_filter = None
+
+        self._incoming_processor = incoming_processor
+        if outgoing_processor is not None:
+            outgoing_processor.send_packet = self.channel_output
+            self.channel_output = outgoing_processor
 
         self.client = HdlcRpcClient(
             self._bytes_queue.get,
             protos,
             default_channels(self.channel_output),
             self.output.write,
-            _incoming_packet_filter_for_testing=incoming_filter).client
+            _incoming_packet_filter_for_testing=incoming_processor).client
 
     def _read_from_socket(self):
         while True:
