@@ -41,6 +41,7 @@ except ImportError:
         os.path.abspath(__file__))))
     import pw_presubmit
 
+import pw_cli.env
 from pw_presubmit import cli, git_repo
 from pw_presubmit.tools import exclude_paths, file_summary, log_run, plural
 
@@ -111,9 +112,10 @@ def clang_format_check(files: Iterable[Path]) -> Dict[Path, str]:
     return _check_files(files, lambda path, _: _clang_format(path))
 
 
-def clang_format_fix(files: Iterable) -> None:
+def clang_format_fix(files: Iterable) -> Dict[Path, str]:
     """Fixes formatting for the provided files in place."""
     _clang_format('-i', *files)
+    return {}
 
 
 def check_gn_format(files: Iterable[Path]) -> Dict[Path, str]:
@@ -125,13 +127,16 @@ def check_gn_format(files: Iterable[Path]) -> Dict[Path, str]:
                                        check=True).stdout)
 
 
-def fix_gn_format(files: Iterable[Path]) -> None:
+def fix_gn_format(files: Iterable[Path]) -> Dict[Path, str]:
     """Fixes formatting for the provided files in place."""
     log_run(['gn', 'format', *files], check=True)
+    return {}
 
 
 def check_bazel_format(files: Iterable[Path]) -> Dict[Path, str]:
     """Checks formatting; returns {path: diff} for files with bad formatting."""
+    errors: Dict[Path, str] = {}
+
     def _format_temp(path: Union[Path, str], data: bytes) -> bytes:
         # buildifier doesn't have an option to output the changed file, so
         # copy the file to a temp location, run buildifier on it, read that
@@ -139,15 +144,27 @@ def check_bazel_format(files: Iterable[Path]) -> Dict[Path, str]:
         with tempfile.TemporaryDirectory() as temp:
             build = Path(temp) / os.path.basename(path)
             build.write_bytes(data)
-            log_run(['buildifier', build], check=True)
+
+            proc = log_run(['buildifier', build], capture_output=True)
+            if proc.returncode:
+                stderr = proc.stderr.decode(errors='replace')
+                stderr = stderr.replace(str(build), str(path))
+                errors[Path(path)] = stderr
             return build.read_bytes()
 
-    return _check_files(files, _format_temp)
+    result = _check_files(files, _format_temp)
+    result.update(errors)
+    return result
 
 
-def fix_bazel_format(files: Iterable[Path]) -> None:
+def fix_bazel_format(files: Iterable[Path]) -> Dict[Path, str]:
     """Fixes formatting for the provided files in place."""
-    log_run(['buildifier', *files], check=True)
+    errors = {}
+    for path in files:
+        proc = log_run(['buildifier', path], capture_output=True)
+        if proc.returncode:
+            errors[path] = proc.stderr.decode()
+    return errors
 
 
 def check_go_format(files: Iterable[Path]) -> Dict[Path, str]:
@@ -157,9 +174,10 @@ def check_go_format(files: Iterable[Path]) -> Dict[Path, str]:
             ['gofmt', path], stdout=subprocess.PIPE, check=True).stdout)
 
 
-def fix_go_format(files: Iterable[Path]) -> None:
+def fix_go_format(files: Iterable[Path]) -> Dict[Path, str]:
     """Fixes formatting for the provided files in place."""
     log_run(['gofmt', '-w', *files], check=True)
+    return {}
 
 
 def _yapf(*args, **kwargs) -> subprocess.CompletedProcess:
@@ -193,9 +211,10 @@ def check_py_format(files: Iterable[Path]) -> Dict[Path, str]:
     return errors
 
 
-def fix_py_format(files: Iterable):
+def fix_py_format(files: Iterable) -> Dict[Path, str]:
     """Fixes formatting for the provided files in place."""
     _yapf('--in-place', *files, check=True)
+    return {}
 
 
 _TRAILING_SPACE = re.compile(rb'[ \t]+$', flags=re.MULTILINE)
@@ -224,8 +243,9 @@ def check_trailing_space(files: Iterable[Path]) -> Dict[Path, str]:
     return _check_trailing_space(files, fix=False)
 
 
-def fix_trailing_space(files: Iterable[Path]) -> None:
+def fix_trailing_space(files: Iterable[Path]) -> Dict[Path, str]:
     _check_trailing_space(files, fix=True)
+    return {}
 
 
 def print_format_check(errors: Dict[Path, str],
@@ -260,13 +280,17 @@ class CodeFormat(NamedTuple):
     extensions: Collection[str]
     exclude: Collection[str]
     check: Callable[[Iterable], Dict[Path, str]]
-    fix: Callable[[Iterable], None]
+    fix: Callable[[Iterable], Dict[Path, str]]
 
 
-C_FORMAT: CodeFormat = CodeFormat(
-    'C and C++',
-    frozenset(['.h', '.hh', '.hpp', '.c', '.cc', '.cpp', '.inc', '.inl']),
-    (r'\.pb\.h$', r'\.pb\.c$'), clang_format_check, clang_format_fix)
+CPP_HEADER_EXTS = frozenset(
+    ('.h', '.hpp', '.hxx', '.h++', '.hh', '.H', '.inc', '.inl'))
+CPP_SOURCE_EXTS = frozenset(('.c', '.cpp', '.cxx', '.c++', '.cc', '.C'))
+CPP_EXTS = CPP_HEADER_EXTS.union(CPP_SOURCE_EXTS)
+
+C_FORMAT: CodeFormat = CodeFormat('C and C++', CPP_EXTS,
+                                  (r'\.pb\.h$', r'\.pb\.c$'),
+                                  clang_format_check, clang_format_fix)
 
 PROTO_FORMAT: CodeFormat = CodeFormat('Protocol buffer', ('.proto', ), (),
                                       clang_format_check, clang_format_fix)
@@ -367,12 +391,22 @@ class CodeFormatter:
 
         return collections.OrderedDict(sorted(errors.items()))
 
-    def fix(self) -> None:
+    def fix(self) -> Dict[Path, str]:
         """Fixes format errors for supported files in place."""
+        all_errors: Dict[Path, str] = {}
         for code_format, files in self._formats.items():
-            code_format.fix(files)
+            errors = code_format.fix(files)
+            if errors:
+                for path, error in errors.items():
+                    _LOG.error('Failed to format %s', path)
+                    for line in error.splitlines():
+                        _LOG.error('%s', line)
+                all_errors.update(errors)
+                continue
+
             _LOG.info('Formatted %s',
                       plural(files, code_format.language + ' file'))
+        return all_errors
 
 
 def _file_summary(files: Iterable[Union[Path, str]], base: Path) -> List[str]:
@@ -390,11 +424,21 @@ def format_paths_in_repo(paths: Collection[Union[Path, str]],
     files = [Path(path).resolve() for path in paths if os.path.isfile(path)]
     repo = git_repo.root() if git_repo.is_repo() else None
 
+    # Implement a graceful fallback in case the tracking branch isn't available.
+    if (base == git_repo.TRACKING_BRANCH_ALIAS
+            and not git_repo.tracking_branch(repo)):
+        _LOG.warning(
+            'Failed to determine the tracking branch, using --base HEAD~1 '
+            'instead of listing all files')
+        base = 'HEAD~1'
+
     # If this is a Git repo, list the original paths with git ls-files or diff.
     if repo:
+        project_root = Path(pw_cli.env.pigweed_environment().PW_PROJECT_ROOT)
         _LOG.info(
             'Formatting %s',
-            git_repo.describe_files(repo, Path.cwd(), base, paths, exclude))
+            git_repo.describe_files(repo, Path.cwd(), base, paths, exclude,
+                                    project_root))
 
         # Add files from Git and remove duplicates.
         files = sorted(
@@ -419,13 +463,19 @@ def format_files(paths: Collection[Union[Path, str]],
     for line in _file_summary(paths, repo if repo else Path.cwd()):
         print(line, file=sys.stderr)
 
-    errors = formatter.check()
-    print_format_check(errors, show_fix_commands=(not fix))
+    check_errors = formatter.check()
+    print_format_check(check_errors, show_fix_commands=(not fix))
 
-    if errors:
+    if check_errors:
         if fix:
-            formatter.fix()
-            # TODO: This should perhaps check that the fixes were successful.
+            _LOG.info('Applying formatting fixes to %d files',
+                      len(check_errors))
+            fix_errors = formatter.fix()
+            if fix_errors:
+                _LOG.info('Failed to apply formatting fixes')
+                print_format_check(fix_errors, show_fix_commands=False)
+                return 1
+
             _LOG.info('Formatting fixes applied successfully')
             return 0
 
