@@ -165,6 +165,7 @@ def gn_crypto_mbedtls_build(ctx: PresubmitContext):
     build.gn_gen(
         ctx.root,
         ctx.output_dir,
+        pw_RUN_INTEGRATION_TESTS=False,
         dir_pw_third_party_mbedtls='"{}"'.format(ctx.package_root / 'mbedtls'),
         pw_crypto_SHA256_BACKEND='"{}"'.format(ctx.root /
                                                'pw_crypto:sha256_mbedtls'),
@@ -179,6 +180,7 @@ def gn_crypto_boringssl_build(ctx: PresubmitContext):
     build.gn_gen(
         ctx.root,
         ctx.output_dir,
+        pw_RUN_INTEGRATION_TESTS=False,
         dir_pw_third_party_boringssl='"{}"'.format(ctx.package_root /
                                                    'boringssl'),
         pw_crypto_SHA256_BACKEND='"{}"'.format(ctx.root /
@@ -195,6 +197,7 @@ def gn_crypto_micro_ecc_build(ctx: PresubmitContext):
     build.gn_gen(
         ctx.root,
         ctx.output_dir,
+        pw_RUN_INTEGRATION_TESTS=False,
         dir_pw_third_party_micro_ecc='"{}"'.format(ctx.package_root /
                                                    'micro-ecc'),
         pw_crypto_ECDSA_BACKEND='"{}"'.format(ctx.root /
@@ -340,6 +343,7 @@ _MODULES_THAT_BUILD_WITH_BAZEL = [
     '//pw_doctor/...',
     '//pw_env_setup/...',
     '//pw_fuzzer/...',
+    '//pw_hdlc/java/...',
     '//pw_hex_dump/...',
     '//pw_i2c/...',
     '//pw_interrupt/...',
@@ -370,7 +374,9 @@ _MODULES_THAT_BUILD_WITH_BAZEL = [
     '//pw_sys_io_baremetal_lm3s6965evb/...',
     '//pw_sys_io_baremetal_stm32f429/...',
     '//pw_sys_io_stdio/...',
+    '//pw_thread/...',
     '//pw_thread_stl/...',
+    '//pw_tokenizer/ts/...',
     '//pw_tool/...',
     '//pw_toolchain/...',
     '//pw_transfer/...',
@@ -389,6 +395,7 @@ _MODULES_THAT_TEST_WITH_BAZEL = [
     '//pw_checksum/...',
     '//pw_cli/...',
     '//pw_containers/...',
+    '//pw_hdlc/java/...',
     '//pw_hex_dump/...',
     '//pw_i2c/...',
     '//pw_libc/...',
@@ -405,7 +412,10 @@ _MODULES_THAT_TEST_WITH_BAZEL = [
     '//pw_status/...',
     '//pw_stream/...',
     '//pw_string/...',
+    '//pw_thread/...',
     '//pw_thread_stl/...',
+    '//pw_tokenizer/ts/...',
+    '//pw_transfer/...',
     '//pw_unit_test/...',
     '//pw_varint/...',
     '//:buildifier_test',
@@ -427,6 +437,18 @@ def bazel_build(ctx: PresubmitContext) -> None:
     build.bazel(ctx, 'build', *_MODULES_THAT_BUILD_WITH_BAZEL)
 
 
+def pw_transfer_integration_test(ctx: PresubmitContext) -> None:
+    """Runs the pw_transfer cross-language integration test only.
+
+    This test is not part of the regular bazel build because it's slow and
+    intended to run in CI only.
+    """
+    build.bazel(
+        ctx, 'test',
+        '//pw_transfer/integration_test:cross_language_integration_test',
+        '--test_output=errors')
+
+
 #
 # General presubmit checks
 #
@@ -532,6 +554,7 @@ _EXCLUDE_FROM_COPYRIGHT_NOTICE: Sequence[str] = (
     r'\.json$',
     r'\.png$',
     r'\.svg$',
+    r'\.xml$',
     # Documentation
     r'\.md$',
     r'\.rst$',
@@ -643,7 +666,7 @@ _GN_SOURCES_IN_BUILD = ('setup.cfg', '.toml', '.rst', '.py',
 
 
 @filter_paths(endswith=(*_GN_SOURCES_IN_BUILD, 'BUILD', '.bzl', '.gn', '.gni'),
-              exclude=['zephyr.*/'])
+              exclude=['zephyr.*/', 'android.*/'])
 def source_is_in_build_files(ctx: PresubmitContext):
     """Checks that source files are in the GN and Bazel builds."""
     missing = build.check_builds_for_files(
@@ -786,7 +809,9 @@ OTHER_CHECKS = (
     gn_clang_build,
     gn_gcc_build,
     gn_pw_system_demo_build,
+    pw_transfer_integration_test,
     renode_check,
+    static_analysis,
     stm32f429i,
 )
 
@@ -803,7 +828,6 @@ _LINTFORMAT = (
 
 LINTFORMAT = (
     _LINTFORMAT,
-    static_analysis,
     pw_presubmit.python_checks.check_python_versions,
     pw_presubmit.python_checks.gn_python_lint,
 )
