@@ -444,7 +444,8 @@ def run(program: Sequence[Callable],
 
         _LOG.info(
             'Checking %s',
-            git_repo.describe_files(repo, repo, base, pathspecs, exclude))
+            git_repo.describe_files(repo, repo, base, pathspecs, exclude,
+                                    root))
 
     if output_directory is None:
         output_directory = root / '.presubmit'
@@ -611,14 +612,6 @@ def filter_paths(endswith: Iterable[str] = '',
     return filter_paths_for_function
 
 
-# TODO(mohrr) Remove after updating downstream projects to use cpp_checks.
-@filter_paths(endswith=('.h'))
-def pragma_once(ctx: PresubmitContext):
-    # pylint: disable=import-outside-toplevel
-    from pw_presubmit import cpp_checks
-    return cpp_checks.pragma_once(ctx)
-
-
 def call(*args, **kwargs) -> None:
     """Optional subprocess wrapper that causes a PresubmitFailure on errors."""
     attributes, command = tools.format_command(args, kwargs)
@@ -649,11 +642,3 @@ def call(*args, **kwargs) -> None:
 
     if process.returncode:
         raise PresubmitFailure
-
-
-# TODO(mohrr) Remove after updating downstream projects to use build.bazel_lint.
-@filter_paths(endswith=('.bzl', '.bazel'))
-def bazel_lint(ctx: PresubmitContext):
-    # pylint: disable=import-outside-toplevel
-    from pw_presubmit import build
-    return build.bazel_lint(ctx)
