@@ -273,6 +273,9 @@ def _generate_info(gen: CodeGenerator, namespace: str,
 
             gen.line('}')
 
+            gen.line(f'using GeneratedClient = ::{namespace}'
+                     f'::pw_rpc::{gen.name()}::{service.name()}::Client;')
+
             gen.method_info_specialization(method)
 
         gen.line('};')
