@@ -78,7 +78,8 @@ def _parse_args() -> argparse.Namespace:
     parser.add_argument(
         '--skip-include-path',
         default=[],
-        nargs='*',
+        action='append',
+        type=str,
         help=('Exclude include paths ending in these paths from clang-tidy. '
               'These paths are switched from -I to -isystem so clang-tidy '
               'ignores them.'))
@@ -104,7 +105,9 @@ def _filter_include_paths(args: Iterable[str],
     for arg in args:
         if arg.startswith('-I'):
             path = Path(arg[2:]).as_posix()
-            if any(path.endswith(f) for f in filters):
+            if any(
+                    path.endswith(f) or re.match(f, str(path))
+                    for f in filters):
                 yield '-isystem' + arg[2:]
                 continue
 
